'''
Copyright (c) 2011 Ansvia. http://www.ansvia.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import os
import re
from string import Template

from IGenerator import IGenerator, GeneratorException


class ScalaGenerator(IGenerator):	
	
	name = "Scala:sbt"
	source_ext = ".scala"
	parameters = [
		{
			'name': 'name',
			'desc': 'Project name.',
			'data_type': 'string',
			'default': 'Hello'
		},
		{
			'name': 'version',
			'desc': 'Project version.',
			'data_type': 'string',
			'default': '0.0.1'
		},
		{
			'name': 'package',
			'desc': 'Package name, ex: com.mycom.myapp',
			'data_type': 'string',
			'default': None
		},
		{
			'name': 'plugins',
			'desc': 'Plugins index list separated by whitespace. ex: 1 2',
			'data_type': 'list',
			'default': [],
			'accept': ['sbt-assembly', 'xsbt-proguard-plugin', 'sbteclipse']
		}
	]

	
	def __init__(self, out_dir, name="Hello", version="0.0.1", main_class=None, package=None, plugins=[]):
		super(ScalaGenerator, self).__init__(out_dir)
		
		self._build_file = "build.sbt"
		
		if not self._build_file:
			raise GeneratorException, "Unsupported kind `%s`" % kind
		
		self.target_name = name
		
		if main_class:
			self.main_class = main_class.title()
		elif name:
			self.main_class = name
		else:
			self.main_class = "Hello"
		
		self.package = package
		self.version = version
		
		_template_build = [
			'name := "$name"',
			'version := "$version"',
			'scalaVersion := "$scala_version"'
		]
		
		if ' ' in plugins:
			plugins = map(lambda x:_get_sbt_plugin(int(x)), plugins.split(' '))
			
			for PClass in plugins:
				p = PClass(self)
				_template_build += p.sbt_build_stuff
		
		self.plugins = plugins
		
		## shorting, all import* place on top
		def _cmp(a, b):
			if a.startswith("import"):
				return -1
			return 1
		
		_template_build = sorted(_template_build, _cmp)
		self._template_build = Template('\n\n'.join(_template_build))
		
		package = ""
		if self.package:
			package = "package " + self.package
		
		self._template_main = Template('''
%(package)s

object $main_class {

	def main(args: Array[String]): Unit = {

		println("Yo World!")
	
	}

}
''' % dict(package = package))

	
	def generate(self):
		build_file = super(ScalaGenerator, self).generate()
		
		if not build_file:
			print "Aborted."
			return False
		
		self.scala_home = os.environ.get('SCALA_HOME')
		if self.scala_home:
			if os.path.exists(self.scala_home) == False:
				raise GeneratorException, "Scala home not exists `%s`" % self.scala_home
			
			if os.path.isdir(self.scala_home) == False:
				raise GeneratorException, "Scala home is not dir `%s`" % self.scala_home
		
		if not self.scala_home:
			## get scala_home
			dirs = [
				'/opt/local/share/scala-2.9',
				'/usr/local/share/scala',
				'/usr/lib/scala',
				'/usr/local/lib/scala',
				'/opt/lib/scala',
				'/usr/share/lib'
			]
			
			for dir in dirs:
				if os.path.exists(dir + "/lib/scala-library.jar"):
					self.scala_home = dir
					break
		
		## Get scala version
		p = os.popen("scala -version")
		rv = p.readline()
		#from dbgp.client import brk; brk()
		version = re.findall(r'version (\d*?\.\d*?\.\d*?)\.', rv)
		if version and len(version) > 0:
			version = version[0]
		p.close()
		
		self.scala_version = version
		
		self.test()

		#main_class_with_package = self.main_class
		
		if self.package:
			main_class_with_package = self.package + "." + self.main_class

		f = open(build_file, "w")
		data = {
			'name': self.target_name,
			#'package': self.package and self.package or '',
			#'main_class': self.main_class,
			'version': self.version,
			'scala_version': self.scala_version,
			#'main_class_with_package': main_class_with_package
		}
		f.write(self._template_build.substitute(**data))
		f.close()
		
		self.build_main_file()
		
		## Process plugins
		for PClass in self.plugins:
			p = PClass(self)
			p.generate()
			p.close()
		
		return True
	
	def build_main_file(self):
		paths = [self._normalize_path(self.out_dir), "src", "main", "scala"]
		
		if self.package:
			paths += self.package.split(".")
		
		out_dir_ = os.path.join(*paths)
		
		if not os.path.exists(out_dir_):
			os.makedirs(out_dir_)
		
		paths.append(self._normalize_source_name(self.main_class))
		
		main_file = os.path.join(*paths)
		
		f = open(main_file, "w")
		f.write(self._template_main.substitute(
			main_class = self.main_class
		))
		f.close()
		
	@staticmethod
	def usage():
		
		parameters = IGenerator.get_help_parameters(ScalaGenerator)
		
		usage_text = '''
Generate Scala project (sbt)

Parameters:

	%(parameters)s

Examples:

	$ progen -p Scala:sbt -o ./hello name=Hello version=0.0.1 main_class="Hello"
''' % dict(parameters=parameters)
		return usage_text
	


	def rollback(self):
		build_file = os.path.join(self._normalize_path(self.out_dir), self._build_file)
		
		if os.path.exists(build_file):
			os.unlink(build_file)
		
		
	def test(self):
		
		if not self.scala_home:
			raise GeneratorException, "Cannot get scala_home, please set it manually by exporting to SCALA_HOME environment"
		
		if os.path.exists(self.scala_home) == False:
			raise GeneratorException, "Scala home not exists `%s`" % self.scala_home
		
		#if self.kind == "exe" and not self.main_class:
		#	raise GeneratorException, "Scala kind `exe` should have main_class, please set it with main_class parameter."

		if not self.scala_version:
			raise GeneratorException, "Cannot find installed scala"

		return True
		
	
class SbtPlugin(object):
	def __init__(self, generator):
		if not isinstance(generator, IGenerator):
			raise GeneratorException, "Invalid ganerator type passed to SbtPlugin __init__"
		self.gen = generator
		self.sbt_file = None
		
	def close(self):
		if self.sbt_file:
			self.sbt_file.close()
			self.sbt_file = None
	
	def generate(self):
		raise GeneratorException, "Plugin.generate not implemented"
	
	def ensure_plugins_sbt(self):
		p = self.gen._normalize_path(self.gen.out_dir) + "/project"
		
		if not os.path.exists(p):
			os.makedirs(p)
		
		p = p + "/plugins.sbt"
		
		self.sbt_file = None
		if not os.path.exists(p):
			self.sbt_file = open(p, "w")
		else:
			self.sbt_file = open(p, "a")
		
		if not self.sbt_file:
			raise GeneratorException, "Cannot create plugins.sbt file"
			
			

class Assembly(SbtPlugin):
	
	name = "sbt-assembly"
	sbt_build_stuff = [
		'import AssemblyKeys._',
		'seq(assemblySettings: _*)'
	]	
	
	def generate(self):
		self.ensure_plugins_sbt()
		self.sbt_file.write('''addSbtPlugin("com.eed3si9n" % "sbt-assembly" % "0.7.2")\n''')
		self.close()


class Proguard(SbtPlugin):
	
	name = "xsbt-proguard-plugin"

	@property
	def sbt_build_stuff(self):
		rv = [
			'seq(ProguardPlugin.proguardSettings :_*)',
			Template('proguardOptions += keepMain("$package.$main_class")').substitute(package = self.gen.package, main_class=self.gen.main_class)
		]
		return rv
	
	def generate(self):
		self.ensure_plugins_sbt()
		
		p = self.gen._normalize_path(self.gen.out_dir) + "/project/project"
		
		if not os.path.exists(p):
			os.makedirs(p)
		
		p = p + "/Build.scala"
		
		f = None
		
		if os.path.exists(p):
			f = open(p, "a")
		else:
			f = open(p, "w")
		
		f.write('''import sbt._
object PluginDef extends Build {
  override def projects = Seq(root)
  lazy val root = Project("plugins", file(".")) dependsOn(proguard)
  lazy val proguard = uri("git://github.com/siasia/xsbt-proguard-plugin.git")
}
''')
		f.close()
		
		self.close()
		

class Sbtclipse(SbtPlugin):
	
	name = "sbteclipse"
	sbt_build_stuff = []
	
	def generate(self):
		self.ensure_plugins_sbt()
		self.sbt_file.write('''
resolvers += Classpaths.typesafeResolver

addSbtPlugin("com.typesafe.sbteclipse" % "sbteclipse" % "1.5.0")\n''')
		self.close()
	


## Register plugins

_sbt_plugins = [
	Assembly,
	Proguard,
	Sbtclipse
]

def _get_sbt_plugin(i):
	index = i - 1
	if index > len(_sbt_plugins):
		raise GeneratorException, "Unknown SbtPlugin index `%d`" % index
	return _sbt_plugins[index]

	



'''
Copyright (c) 2011 Ansvia. http://www.ansvia.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import os
from string import Template

from IGenerator import IGenerator, GeneratorException


class ScalaGenerator(IGenerator):	
	
	name = "Scala"
	source_ext = ".scala"
	parameters = [
		{
			'name': 'kind',
			'data_type': 'string',
			'desc': '''Project kind, currently support `exe` and `lib`.
			`exe` for create executable jar target.
			`lib` for create java library.''',
			'default': 'exe'
		},
		{
			'name': 'target_name',
			'desc': 'Project name.',
			'data_type': 'string',
			'default': 'Hello'
		},
		{
			'name': 'sources',
			'desc': 'Scala sources, separated by whitespace.',
			'data_type': 'list',
			'default': ['Hello.scala']
		},
		{
			'name': 'main_class',
			'desc': 'if project kind == `exe` then specify this one for class entry point.',
			'data_type': 'string',
			'default': "Hello"
		},
		{
			'name': 'package',
			'desc': 'Package name, ex: com.mycom.myapp',
			'data_type': 'string',
			'default': None
		}
	]
	
	def __init__(self, out_dir, kind="exe", target_name="Hello.jar", sources=["Hello.scala"], main_class="Hello", package=None):
		super(ScalaGenerator, self).__init__(out_dir)
		
		if kind == "exe":
			self._build_file = "build.xml"
		
		if not self._build_file:
			raise GeneratorException, "Unsupported kind `%s`" % kind
		
		if kind not in ['exe', 'lib']:
			raise GeneratorException, "Invalid kind, only support: `exe` and `lib`." 
		
		self.kind = kind
		self.target_name = target_name
		self.main_class = main_class.title()
		self.package = package
		
		if isinstance(sources, (str, unicode)):
			sources = sources.split(" ")
		
		self.sources = sources
		self._template_build = Template('''
<project default="run" name="$main_class">
	<property name="project.dir" value="." />
	<property name="main.class" value="$main_class_with_package" />
	<property name="scala.home" value="$scala_home" />
	<property name="build.dir" value="$${project.dir}/build" />
	<property name="source.dir" value="$${project.dir}/src" />
	
    <target name="init">
    	
    	<!-- scala libraries for classpath definitions -->
    	<property name="scala-library.jar" value="$${scala.home}/lib/scala-library.jar" />
    	<property name="scala-compiler.jar" value="$${scala.home}/lib/scala-compiler.jar" />
    	
        <!-- classpath for the compiler task definition -->
        <path id="scala.classpath">
          <pathelement location="$${scala-compiler.jar}" />
          <pathelement location="$${scala-library.jar}" />
        </path>

        <!-- classpath for project build -->
    	<path id="build.classpath">
    		<pathelement location="$${scala-library.jar}"/>
    		<fileset dir="$${project.dir}/lib">
    			<include name="*.jar"/>
			</fileset>
    		<pathelement location="$${build.dir}/classes" />
    	</path>
    	
    	<taskdef resource="scala/tools/ant/antlib.xml">
	      <classpath refid="scala.classpath" />
	    </taskdef>
    	
    	<mkdir dir="$${build.dir}" />
    	<mkdir dir="$${source.dir}" />
    	<mkdir dir="$${project.dir}/lib" />

    </target>

    <target name="build" depends="init" description="Build class">
        <mkdir dir="$${build.dir}/classes" />
    	<scalac classpathref="build.classpath" deprecation="on"
    	       destdir="$${build.dir}/classes"
    	       force="never" srcdir="$${source.dir}">
    	      <include name="**/*.scala">
    	    </include>
    	</scalac>
    </target>

	
	  <!-- delete compiled files -->
	  <target description="clean" name="clean">
		<delete includeemptydirs="true">
	  		<fileset dir="$${build.dir}" />
	  	</delete>
	    <delete dir="$${project.dir}/doc" />
	  </target>
	
	  <!-- run program -->
	  <target depends="build" description="run" name="run">
	    <java classname="$${main.class}" classpathref="build.classpath" />
	  </target>
	
	<!-- ================================= 
          target: $target_name              
         ================================= -->
    <target name="$target_name" depends="build" description="Create $target_name file">
        <mkdir dir="$${build.dir}/jar" />
    	<copy file="$${scala-library.jar}" todir="$${project.dir}/lib/" />
    	<path id="jar.class.path">
    		<fileset dir="$${project.dir}">
    			<include name="lib/**/*.jar" />
    		</fileset>
    	</path>
    	<pathconvert dirsep="/" pathsep=" " property="jar.classpath">
    	      <path refid="jar.class.path"></path>
    	      <map from="$${basedir}$${file.separator}lib" to="lib" />
    	</pathconvert>
    	<jar basedir="$${build.dir}/classes" destfile="$${build.dir}/jar/$${ant.project.name}.jar">
    		<zipgroupfileset dir="$${project.dir}/lib" includes="*.jar" />
    		<manifest>
    			<attribute name="Main-Class" value="$${main.class}" />
    			<attribute name="Class-Path" value="$${jar.classpath}" />
    		</manifest>
    	</jar>
    </target>

</project>
		'''.strip() + "\n")
		
		package = ""
		if self.package:
			package = "package " + self.package
		
		self._template_main = Template('''
%(package)s

object $main_class {

	def main(args: Array[String]): Unit = {

		println("Yo World!")
	
	}

}
''' % dict(package = package))

	
	def generate(self):
		#build_file = super(ScalaGenerator, self).generate()
		
		self.scala_home = os.environ.get('SCALA_HOME')
		if self.scala_home:
			if os.path.exists(self.scala_home) == False:
				raise GeneratorException, "Scala home not exists `%s`" % self.scala_home
			
			if os.path.isdir(self.scala_home) == False:
				raise GeneratorException, "Scala home is not dir `%s`" % self.scala_home
		
		if not self.scala_home:
			## get scala_home
			dirs = [
				'/opt/local/share/scala-2.9',
				'/usr/local/share/scala',
				'/usr/lib/scala',
				'/usr/local/lib/scala',
				'/opt/lib/scala'
			]
			
			for dir in dirs:
				if os.path.exists(dir + "/lib/scala-library.jar"):
					self.scala_home = dir
					break
		
		self.test()

		main_class_with_package = self.main_class
		
		if self.package:
			main_class_with_package = self.package + "." + self.main_class

		f = open(self._build_file, "w")
		data = {
			'target_name': self.target_name,
			'package': self.package and self.package or '',
			'main_class': self.main_class,
			'scala_home': self.scala_home,
			'main_class_with_package': main_class_with_package
		}
		f.write(self._template_build.substitute(**data))
		f.close()
		
		self.build_main_file()
		
		return True
	
	def build_main_file(self):
		paths = [self._normalize_path(self.out_dir), "src"]
		
		if self.package:
			paths += self.package.split(".")
		
		out_dir_ = os.path.join(*paths)
		
		if not os.path.exists(out_dir_):
			os.makedirs(out_dir_)
		
		paths.append(self._normalize_source_name(self.main_class))
		
		main_file = os.path.join(*paths)
		
		f = open(main_file, "w")
		f.write(self._template_main.substitute(
			main_class = self.main_class
		))
		f.close()

	def rollback(self):
		build_file = os.path.join(self._normalize_path(self.out_dir), self.__build_file)
		
		if os.path.exists(build_file):
			os.unlink(build_file)
		
		
	def test(self):
		if self.kind not in ['exe', 'lib']:
			raise GeneratorException, "Invalid kind, only support: `cmd` and `pkg`."
		
		if isinstance(self.sources, (tuple, list)) == False:
			raise GeneratorException, "Invalid sources type, should be tuple or list"
	
		if not self.scala_home:
			raise GeneratorException, "Cannot get scala_home, please set it manually by exporting to SCALA_HOME environment"
		
		if os.path.exists(self.scala_home) == False:
			raise GeneratorException, "Scala home not exists `%s`" % self.scala_home
		
		if self.kind == "exe" and not self.main_class:
			raise GeneratorException, "Scala kind `exe` should have main_class, please set it with main_class parameter."

		return True
		
	@staticmethod
	def usage():
		parameters = IGenerator.get_help_parameters(ScalaGenerator)
		
		usage_text = '''
Generate Scala project

Parameters:

	%(parameters)s

Examples:

	$ progen -p Scala -o ./hello kind=exe target_name=hello.jar sources="Hello.scala" main_class="Hello"
''' % dict(parameters=parameters)
		return usage_text

	






'''
Copyright (c) 2011 Ansvia. http://www.ansvia.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import os
from string import Template

from IGenerator import IGenerator, GeneratorException


class GolangGenerator(IGenerator):	
	
	name = "Golang"
	source_ext = ".go"
	parameters = [
		{
			'name': 'kind',
			'data_type': 'string',
			'desc': '''Project kind, can be `cmd` or `pkg`.
		`cmd` for create executable project,
		`pkg` for create package / go lib.''',
			'default': 'cmd'
		},
		{
			'name': 'target_name',
			'data_type': 'string',
			'desc': 'Output target file name',
			'default': 'hello'
		},
		{
			'name': 'sources',
			'data_type': 'list',
			'desc': 'Golang sources, separated by whitespace.',
			'default': ["hello.go"]
		}
	]

	
	def __init__(self, out_dir, kind="cmd", target_name="hello", sources=["hello.go"]):
		super(GolangGenerator, self).__init__(out_dir)
		self._build_file = "Makefile"
		
		if kind not in ['cmd', 'pkg']:
			raise GeneratorException, "Invalid kind, only support: `cmd` and `pkg`." 
		
		self.kind = kind
		self.target_name = target_name
		
		if isinstance(sources, (unicode, str)):
			sources = sources.split(" ")
		
		self.sources = sources
		self._template = Template('''
include $$(GOROOT)/src/Make.inc

TARG=$target_name
DEPS=

GOFILES=$sources

include $$(GOROOT)/src/Make.$kind
		'''.strip() + "\n")
		self._template_main = Template('''
package main

import (
	"fmt"
)

func main() {
	fmt.Println("Yo World!")
}
'''.strip() + "\n")
	
	def generate(self):
		build_file = super(GolangGenerator, self).generate()
	
		self.test()

		f = open(build_file, "w")
		data = {
			'target_name': self.target_name,
			'sources': " ".join(self.sources),
			'kind': self.kind
		}
		#from dbgp.client import brk; brk()
		f.write(self._template.substitute(**data))
		f.close()
		
		main_file = os.path.join(self._normalize_path(self.out_dir), self.target_name)
		
		main_file = self._normalize_source_name(main_file)
		
		f = open(main_file, "w")
		f.write(self._template_main.substitute())
		f.close()
		
		return True
	

	def rollback(self):
		build_file = os.path.join(self._normalize_path(self.out_dir), self.__build_file)
		
		if os.path.exists(build_file):
			os.unlink(build_file)
		
		
	def test(self):
		if self.kind not in ['cmd', 'pkg']:
			raise GeneratorException, "Invalid kind, only support: `cmd` and `pkg`."
		
		if isinstance(self.sources, (tuple, list)) == False:
			raise GeneratorException, "Invalid sources type, should be tuple or list"

		return True


	@staticmethod
	def usage():
		parameters = IGenerator.get_help_parameters(GolangGenerator)
		
		usage_text = '''
Generate Golang project.

Parameters:
	
	%(parameters)s
	
Examples:
	
	$ progen -p Golang -o ./ kind=cmd target_name=hello sources="hello.go"
	
''' % dict(parameters=parameters)
	
		return usage_text
		
	

	






'''
Copyright (c) 2011 Ansvia. http://www.ansvia.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import os
import re

class GeneratorException(Exception):
	pass

class IGenerator(object):
	
	name = ""
	source_ext = ""
	usage = ""
	
	def __init__(self, out_dir):
		self.out_dir = out_dir
		self._build_file = None
	
	def generate(self):
		if os.path.isdir(self.out_dir) != True:
			if os.path.isfile(self.out_dir) != True:
				os.makedirs(self.out_dir)
			else:
				raise GeneratorException, "Already exists as file `%s`" % self.out_dir
		
		build_file = os.path.join(self._normalize_path(self.out_dir), self._build_file)
		
		if os.path.exists(build_file):
			print "Build file already exists `%s`" % build_file,
			ui = raw_input("Remove? [y/n] ")
			if ui == "y":
				os.unlink(build_file)
				if os.path.exists(build_file):
					raise GeneratorException, "Build file already exists `%s` (cannot remove)" % build_file
			else:
				return None
		
		return build_file
	
	def set_build_file(self, build_file):
		self.__build_file = build_file
	
	def _normalize_path(self, path):
		rv = path.strip("")
		rv = re.sub("/+$", "", rv)
		return rv
	
	def _normalize_source_name(self, path):
		if path.endswith(self.source_ext):
			return path
		return path + self.source_ext
	
	@staticmethod
	def get_help_parameters(Cls):
		rv = []
		params = Cls.parameters
		for p in params:
			if p.has_key('default') and p['default']:
				rv.append('%s -- {%s} %s, default %s.' % (p['name'], p['data_type'], p['desc'], p['default']))
			else:
				rv.append('%s -- {%s} %s' % (p['name'], p['data_type'], p['desc']))
			
			if p.has_key('accept'):
				rv.append("\tsupport: %s.\n" % ", ".join(p['accept']))
			
		return "\n\t".join(rv)
	
	@staticmethod
	def usage():
		raise GeneratorException, "Not implemented"
	
	def build_main_file(self):
		raise GeneratorException, "Not implemented"

	def rollback(self):
		"""This called when gerenator fail.
		"""
		raise GeneratorException, "Not implemented"

	def __repr__(self):
		return "<Generator[%s]>" % str(self.name)

	def test(self):
		raise GeneratorException, "Not implemented"






'''
Copyright (c) 2011 Ansvia. http://www.ansvia.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

from optparse import OptionParser

import sys

from generators import *

VERSION = "0.0.4"

_generators = [
	golang.GolangGenerator,
	scala.ScalaGenerator,
	scala_sbt.ScalaGenerator
]


def get_generator(name):
	for g in _generators:
		if g.name == name:
			return g
	return None

def list_generators():
	for g in _generators:
		print "* " + g.name
		print "  `help %s` to show %s's project spec details" % (g.name, g.name )
		print ""

parser = OptionParser()

parser.add_option("-p", "--project", dest="project", help="Project for. Show supported generators using `-t` parameter.")
parser.add_option("-o", "--out-dir", dest="out_dir", help="Output directory")
parser.add_option("-i", "--interactive", dest="interactive", action="store_true", help="Interactive mode")
parser.add_option("-t", dest="list_generators", action="store_true", help="Show supported generators.")

usage = '''
To get usage details on every supported project,
execute `help` followed by project name, example:

	$ progen help Golang
	
'''

def main():
	
	print "Ansprogen " + VERSION
	
	opts, args = parser.parse_args()
	
	if len(args) == 2:
		opt, proj = args
		if opt == "help":
			gen = get_generator(proj)
			if gen:
				print gen.usage()
				return 0
			else:
				print "Unknown help for project `%s`" % proj
				return 3
	
	if opts.list_generators:
		print ""
		print "Supported generators:\n"
		list_generators()
		return
	
	if not opts.project:
		print "No project name, please specify it with `-p` or `--project`"
		parser.print_help()
		print usage
		return 1

	if not opts.out_dir:
		print "No output dir, please specify it with `-o` or `--out-dir`"
		parser.print_help()
		print usage
		return 1

	if not opts.interactive and not opts.out_dir:
		print "No out dir, please specify it with `-o` or `--out-dir`"
		parser.print_help()
		print usage
		return 1
	
	Generator = get_generator(opts.project)
	
	if not Generator:
		print "Unknown project `%s`" % opts.project
		parser.print_usage()
		print usage
		return 2
	

	params = dict(map(lambda x: x.split("="), args))
	params['out_dir'] = opts.out_dir
	
	#from dbgp.client import brk; brk()
	
	if opts.interactive:
		print "Creating %s project" % Generator.name
		print "Interactive mode."
		i_params = Generator.parameters
		for p in i_params:
			name = p['name']
			
			if p.has_key('desc'):
				print "   " + p['desc']
				
			if p.has_key('accept'):
				accept = p['accept']
				for i, a in enumerate(accept):
					accept[i] = "%d: %s" % ((i+1), a)
				print "   supported: " + ', '.join(accept)
			
			if isinstance(p['default'], (str, unicode)):
				default = p.has_key('default') and p['default'] or ''
			else:
				default = ''
			
			param = raw_input(" -> " + name + ' [' + default + '] ' + ": ")
			
			if len(param.strip()) == 0:
				param = default
			
			params[name] = param
	
	#print params
	
	#from dbgp.client import brk; brk()
	gen = Generator(**params)
	
	rv = gen.generate()
	
	if rv != True:
		gen.rollback()
		return 4
	
	print "Output: `%s`" % opts.out_dir
	print "Done."

	return 0



if __name__ == '__main__':
	sys.exit(main())






